/*******************************************************************************
 * platformbase.h
 *
 * This module contains all defines, typedefs, and prototypes for the
 * C++ interface of the abstract PlatformBase class.
 *
 * from Persistence of Vision Ray Tracer ('POV-Ray') version 3.7.
 * Copyright 1991-2003 Persistence of Vision Team
 * Copyright 2003-2010 Persistence of Vision Raytracer Pty. Ltd.
 * ---------------------------------------------------------------------------
 * NOTICE: This source code file is provided so that users may experiment
 * with enhancements to POV-Ray and to port the software to platforms other
 * than those supported by the POV-Ray developers. There are strict rules
 * regarding how you are permitted to use this file. These rules are contained
 * in the distribution and derivative versions licenses which should have been
 * provided with this file.
 *
 * These licences may be found online, linked from the end-user license
 * agreement that is located at http://www.povray.org/povlegal.html
 * ---------------------------------------------------------------------------
 * POV-Ray is based on the popular DKB raytracer version 2.12.
 * DKBTrace was originally written by David K. Buck.
 * DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
 * ---------------------------------------------------------------------------
 * $File: //depot/povray/spec-3.7/source/base/platformbase.h $
 * $Revision: #2 $
 * $Change: 5047 $
 * $DateTime: 2010/06/30 07:58:31 $
 * $Author: thorsten $
 *******************************************************************************/

#ifndef PLATFORMBASE_H
#define PLATFORMBASE_H

#include "base/types.h"
#include "base/fileinputoutput.h"

#include <string>

namespace pov_base
{

#define POV_PLATFORM_BASE PlatformBase::GetPlatformBaseReference()

class PlatformBase
{
	public:
		PlatformBase() { self = this; };
		virtual ~PlatformBase() { self = NULL; };

		virtual IStream *CreateIStream(const unsigned int stype) = 0;
		virtual OStream *CreateOStream(const unsigned int stype) = 0;

		virtual UCS2String GetTemporaryPath() = 0;
		virtual UCS2String CreateTemporaryFile() = 0;
		virtual void DeleteTemporaryFile(const UCS2String& filename) = 0;

		virtual bool ReadFileFromURL(OStream *file, const UCS2String& url, const UCS2String& referrer = UCS2String()) = 0;

		static PlatformBase& GetPlatformBaseReference() { return *self; };
	private:
		static PlatformBase *self;
};

}

#endif
