/*******************************************************************************
 * defaultplatformbase.cpp
 *
 * This module contains the C++ DefaultPlatformBase class.
 *
 * from Persistence of Vision Ray Tracer ('POV-Ray') version 3.7.
 * Copyright 1991-2003 Persistence of Vision Team
 * Copyright 2003-2010 Persistence of Vision Raytracer Pty. Ltd.
 * ---------------------------------------------------------------------------
 * NOTICE: This source code file is provided so that users may experiment
 * with enhancements to POV-Ray and to port the software to platforms other
 * than those supported by the POV-Ray developers. There are strict rules
 * regarding how you are permitted to use this file. These rules are contained
 * in the distribution and derivative versions licenses which should have been
 * provided with this file.
 *
 * These licences may be found online, linked from the end-user license
 * agreement that is located at http://www.povray.org/povlegal.html
 * ---------------------------------------------------------------------------
 * POV-Ray is based on the popular DKB raytracer version 2.12.
 * DKBTrace was originally written by David K. Buck.
 * DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
 * ---------------------------------------------------------------------------
 * $File: //depot/povray/spec-3.7/source/frontend/defaultplatformbase.cpp $
 * $Revision: #2 $
 * $Change: 5047 $
 * $DateTime: 2010/06/30 07:58:31 $
 * $Author: thorsten $
 *******************************************************************************/

// configbase.h must always be the first POV file included within base *.cpp files
#include "base/configbase.h"

#include "defaultplatformbase.h"

#include "povms.h"
#include "pov_err.h"

// this must be the last file included
#include "base/povdebug.h"

namespace pov_base
{

DefaultPlatformBase::DefaultPlatformBase()
{
}

DefaultPlatformBase::~DefaultPlatformBase()
{
}

IStream *DefaultPlatformBase::CreateIStream(const unsigned int stype)
{
	return new IStream(stype);
}

OStream *DefaultPlatformBase::CreateOStream(const unsigned int stype)
{
	return new OStream(stype);
}

UCS2String DefaultPlatformBase::GetTemporaryPath()
{
	return ASCIItoUCS2String("/tmp/");
}

UCS2String DefaultPlatformBase::CreateTemporaryFile()
{
	static int cnt = 0;
	char buffer[32];

	cnt++;
	sprintf(buffer, "/tmp/pov%08x.dat", cnt);

	FILE *f = fopen(buffer, "wb");
	if(f != NULL)
		fclose(f);

	return UCS2String(ASCIItoUCS2String(buffer));
}

void DefaultPlatformBase::DeleteTemporaryFile(const UCS2String& filename)
{
	remove(UCS2toASCIIString(filename).c_str());
}

bool DefaultPlatformBase::ReadFileFromURL(OStream *, const UCS2String&, const UCS2String&)
{
	return false;
}


}
